/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class TreeMultisetTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testCreate() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testCreate();
}

public void testCreateFromIterable() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testCreateFromIterable();
}

public void testCreateWithComparator() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testCreateWithComparator();
}

public void testCustomComparator() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testCustomComparator();
}

public void testDegenerateComparator() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testDegenerateComparator();
}

public void testElementSetSortedSetMethods() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testElementSetSortedSetMethods();
}

public void testElementSetSubsetClear() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testElementSetSubsetClear();
}

public void testElementSetSubsetRemove() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testElementSetSubsetRemove();
}

public void testElementSetSubsetRemoveAll() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testElementSetSubsetRemoveAll();
}

public void testElementSetSubsetRetainAll() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testElementSetSubsetRetainAll();
}

public void testNullAcceptingComparator() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testNullAcceptingComparator();
}

public void testSubMultisetSize() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testSubMultisetSize();
}

public void testToString() throws Exception {
  com.google.common.collect.TreeMultisetTest testCase = new com.google.common.collect.TreeMultisetTest();
  testCase.testToString();
}
}
